package data;

import java.awt.*;
import java.applet.*;
import java.util.*;
import java.io.*;
import java.net.*;
import NDimViewer;
import graphoutput.*;
import DynSysInfo;

/*
 *
 * DataDialog
 *
 */
public class DataDialog extends Frame
{
	// Verndert die Gleichung mit dem angegebenen Index
	// und zeigt sie an

	boolean editEquation( int nIndex, String sEquation, double nOrigin ) 
	{
		if( nIndex >= 0 )
		{
			// in diesen Platz die neue Gleichung eintragen
			sBody.setElementAt(sEquation,nIndex);
			elementOrigin.setElementAt(new Double(nOrigin),nIndex);
		}
		
		updateEquationList( nIndex );

		return true;
	}


	// Speicherung des Gleichungssystemes

	Vector		sId = new Vector();			// Namen der Variablen
	Vector		sBody = new Vector();		// Gleichung

	public DataElement elementOrigin = new DataElement();

	// bertrgt die Gleichungen zur Anzeige in die
	// Liste im Dialog
	void updateEquationList( int index )
	{
		lstEquation.clear();

		for( int i = 0; i < sId.size(); i++ )
		{
			String sBuffer;

			sBuffer  = (String)sId.elementAt(i);
			sBuffer += "=";
			sBuffer += (String)sBody.elementAt(i);

			if( ((String)sId.elementAt(i)).length() > 1 )
			{
				Double x = (Double)elementOrigin.elementAt(i);

				sBuffer += " [";
				// sBuffer += elementOrigin.toString();
				sBuffer += x.toString();
				sBuffer += "]";
			}

			lstEquation.addItem( sBuffer );
		}

		if( index >= 0 ) lstEquation.select( index );

	}

	// sucht, ob fr diese Variable bereits eine 
	// Gleichung eingegeben wurde
	int findId( String sName )
	{
		if( sName.length() > 1 )
			sName = sName.substring( 0, 1 );
		
		for( int i = 0; i < sId.size(); i++ )
		{
			String sCompare = (String)sId.elementAt(i);

			if( sCompare.length() > 1 )
				sCompare = sCompare.substring(0,1);

			if( sName.equalsIgnoreCase( sCompare ) )
				return i;
		}

		return -1;
	}
	
	// nimmt eine zustzliche Gleichung auf

	boolean addEquation( String sName, String sEquation, double nOrigin ) 
	{
		// gibt's diese Variable bereits
		int i = findId( sName );
		
		if( i < 0 )
		{
			// also einen leeren Platz generieren
			sId.addElement( sName );
			sBody.addElement( sEquation );
			elementOrigin.addElement( new Double(nOrigin) );
		}
		else
		{
			// in diesen Platz die neue Gleichung eintragen
			sId.setElementAt(sName, i);
			sBody.setElementAt(sEquation,i);
			elementOrigin.setElementAt( new Double(nOrigin),i);
		}
		
		updateEquationList( i );

		return true;
	}

	// Entfernt aus dem bergebenen Datenelement alle
	// Stellen, die nicht zu einer Gleichung, sondern
	// nur zu einer Hilfsgleichung (0.0) gehren

	DataElement trimOrigin( DataElement elementOrigin )
	{
		DataElement result = new DataElement();

		for( int i = 0; i < elementOrigin.size(); i++ )
			if( ((String)sId.elementAt(i)).length() > 1 )
				result.addElement( elementOrigin.elementAt(i) );

		return result;
	}

	// Erzeugt aus dem Gleichungssystem ein Datenelement

	public DataList parseSystem()
	{
		DataList data = new DataList(applet.go);

		// damit ich mitverfolgen kann,
		// ob das Applet arbeitet, bietet es die Reporter
		// Schnittstelle
		data.setReporter( (Reporter)applet );

		// Quelle Eintragen
		data.nOrigin = DataList.SYSTEM;

		// Schrittweite h sichern
		data.nStep = Double.valueOf( txtStep.getText() ).doubleValue();

		// Iterationsanzahl eintragen
		data.nIterations = Long.valueOf( txtIterations.getText() ).longValue();

		// Nherungsmethode 
		if( dataMethode.getCurrent().getLabel().equals( "Euler" ) ) 
			data.nMethode = DataList.EULER;
		else
			data.nMethode = DataList.RUNGEKUTTA;

		// Anzeigepunkte sichern
		data.nDisplay = Long.valueOf( txtDisplay.getText() ).longValue();

		// Gleichungen eintragen:
		// Gleichungen sind alle Ids von der Form x'=

		// Hilfsgleichungen bertragen:
		// Hilfsgleichungen sind alle Ids der Form x=
		int i;

		data.clearEquation(); 
		data.clearHelper();

		for( i = 0; i < sId.size(); i++ )
		{
			if( ((String)sId.elementAt(i)).length() > 1 )
				data.addEquation( (String)sId.elementAt(i), (String)sBody.elementAt(i) );
			else
				data.addHelper( (String)sId.elementAt(i), (String)sBody.elementAt(i) );
		}
		// Startpunkt fr Iteration bergeben:
		// im lokalen elementOrigin sind aber auch 
		// Positionen fr die Hilfsgleichungen (0.0) 
		// enthalten. Die drfen allerdings nicht ins
		// Datenelement
		data.elementOrigin = trimOrigin(elementOrigin);

		// Daten generieren
		data.generateData();

		// nach dem Aufbauen einer DataList die Update-Methode
		// aufrufen: min, max, etc. Berechnung
		data.update();

		return data;
	}

	// holt die letzte Eingabe des Gleichungssystems
	// aus dem Datenelement heraus

	void initSystem( DataList data ) 
	{
		if( data == null ) return; 

		// Schrittweite h
		txtStep.setText( Double.toString( data.nStep ) );

		// Iterationsanzahl
		txtIterations.setText( Long.toString( data.nIterations ) );

		// Methode fr Nherung
		if( data.nMethode == DataList.EULER )
			dataMethode.setCurrent( dataMethodeEuler );
		else
			dataMethode.setCurrent( dataMethodeRungeKutta );

		// Anzeigepunkte eintragen
		txtDisplay.setText( Long.toString( data.nDisplay ) );

		// Hole die Gleichungssysteme...
		int i;

		for( i = 0; i < data.countEquation(); i++ )
		{
			sId.addElement( data.getName( i ) + "'" );
			sBody.addElement( data.getEquation( i ) );
		}

		for( i = 0; i < data.countHelper(); i++ )
		{
			sId.addElement( data.getHelperId( i ) );
			sBody.addElement( data.getHelperBody( i ) );
		}

		// Ursprung:
		// in der internen Version des Ursprungsvektors 
		// werden fr alle Gleichungen Eintrge gespeichert;
		// d.h. fr die Hilfsgleichungen werden 0.0 
		// Positionen mitgespeichert
		if( data.elementOrigin != null )
		{
			elementOrigin = (DataElement)data.elementOrigin.clone();

			for( i = elementOrigin.size(); i < sId.size(); i++ )
				elementOrigin.addElement( new Double( 0.0 ) );

		}

		// ...und zeige sie in der Liste an
		updateEquationList(-1);

	}

	Button btnAdd;
	Button btnDelete;
	Button btnEdit;

	java.awt.List lstEquation;

	TextField txtStep;
	TextField txtIterations;
	TextField txtDisplay;

	CheckboxGroup dataMethode;

	Checkbox dataMethodeEuler;
	Checkbox dataMethodeRungeKutta;


	// zeigt die enthaltenen Daten eines Datenfeldes im Textfeld an
	void initText( DataList dataList ) 
	{
		if( dataList == null ) return;

		txtText.setText( dataList.getText() );
	}


	// Hilfsfunktion, die ermittelt, ob eine eingelesene
	// Datei Mepunkte oder ein Gleichungssystem enthlt

	// Bei einer Datei mit Mewerten (isText=true) ist das
	// erste Zeichen eine Zahl bzw. ein "."

	protected boolean isText( String s )
	{
		// Whitespace entfernen
		s.trim();

		String sTest = ".0123456789";

		return sTest.indexOf( s.substring( 0, 1 ) ) > -1;
	}

	// Hilfsfunktione, die ein Gleichungssystem aus einer
	// Datei liest

	// der Inhalt der Datei wird im String s bergeben

	protected String parseURLSystemSection( String s, String section )
	{
		// wo beginnt die gewnschte Sektion
		int	nStart = s.indexOf( section ) + section.length();
		if( nStart < 0 ) return "";

		// extrahiere diesen Teil
		String sTemp = s.substring( nStart );

		// wo ist das Ende (der Beginn der nchsten Sektion)
		int nEnd = sTemp.indexOf( "[" );

		String sResult = sTemp; 

		// ist berhaupt eine nchste Sektion vorhanden
		if( nEnd >= 0 ) 
			sResult = sTemp.substring( 0, nEnd );

		sResult.trim();

		return sResult;
	}

	protected String parseURLSystemKey( String s, String key )
	{
		int nPos = s.indexOf( key + "=" );

		// dieser Key kommt eventuell gar nicht vor
		if( nPos < 0 ) return "";

		// kommt er vor, dann berechne den Anfang des Wertes
		// dieses Schlssels
		nPos += key.length() + 1;

		String sTemp = s.substring( nPos );

		// jetzt den Wert bis zum Ende der Zeile bzw. Ende
		// des Strings ermitteln
		int nEnd = sTemp.indexOf( "\n" );

		if( nEnd < 0 ) return sTemp;

		return sTemp.substring( 0, nEnd );
	}

	protected DataList parseURLSystem( String s )
	{
		MsgBox msg;
		DataList dataList = new DataList(applet.go);

		// zuerst wird die Sektion [Start] gesucht und extrahiert
		String sStart = parseURLSystemSection( s, "[Start]" );

		// danach wird nach einer Sektion [Equations] gesucht und extrahiert
		StringTokenizer sEquations = new StringTokenizer( parseURLSystemSection( s, "[Equations]" ), "\n" );

		dataList.clearEquation();
		dataList.clearHelper();

		elementOrigin = new DataElement();

		while( sEquations.hasMoreElements()	)
		{
			String line = sEquations.nextToken(); line.trim();

			// alle nicht Leerzeilen werden behandelt
			if( line.length() > 0 )
			{
				// zuerst die Zeile in Variable und Ausdruck teilen
				int nPos = line.indexOf( "=" );

				if( nPos >= 0 )
				{
					String sId = line.substring( 0, nPos );
					String sBody = line.substring( nPos+1 );

					// Ist es eine Gleichung oder ein Parameter
					// Unterschieden an x' (2 Zeichen) oder x (1 Zeichen)
					if( sId.length() > 1 )
					{
						dataList.addEquation( sId, sBody );

						// den dazugehrigen Startpunkt suchen
						String sOrigin = parseURLSystemKey(sStart,sId.substring(0,1));
						Double nOrigin = new Double( 0 );

						if( sOrigin.length() > 0 )
							nOrigin = Double.valueOf(sOrigin);

						elementOrigin.addElement( nOrigin );
					}
					else
						dataList.addHelper( sId, sBody );

				}
			}
		}

		dataList.elementOrigin = elementOrigin;

		// zuletzt wird die Sektion [Options] ausgewertet
		String sOptions = parseURLSystemSection( s, "[Options]" );

		String h;

		// ist der Parameter Steps vorhanden
		h = parseURLSystemKey( sOptions, "Steps" );

		if( h.length() > 0 ) 
			try { dataList.nIterations = Long.valueOf( h ).longValue(); }
			catch( Exception e ) {}

		// ist der Parameter Stepwidth vorhanden
		h = parseURLSystemKey( sOptions, "Stepwidth" );
		if( h.length() > 0 ) 
			try { dataList.nStep = Double.valueOf( h ).doubleValue(); }
			catch( Exception e ) {}

		// ist der Parameter DisplaySteps vorhanden
		h = parseURLSystemKey( sOptions, "DisplaySteps" );
		if( h.length() > 0 ) 
			try { dataList.nDisplay = Long.valueOf( h ).longValue(); }
			catch( Exception e ) {}

		// ist der Parameter Methode vorhanden
		h = parseURLSystemKey( sOptions, "Method" );
		
		if( h.equalsIgnoreCase( "RUNGE-KUTTA" ) )
			dataList.nMethode = DataList.RUNGEKUTTA; 

		return dataList;
	}

	// Liest eine Datei ber einen URL ein

	DataList parseURL()
	{
		MsgBox msg;

		// in diese Variable wird der Inhalt des Files gelesen
		String sBuffer = "";

		try
		{
			URL url = new URL( applet.getCodeBase(), txtURL.getText() );

			InputStream is = url.openStream();
			DataInputStream dis = new DataInputStream( is );

			String sLine;

			while( (sLine = dis.readLine()) != null )
				sBuffer += sLine + "\n";

		}
		catch( Exception e )
		{
			msg = new MsgBox( this, "Wrong URL", "The file can't be read." );
			return null;
		}

/*		// Erzeugen des File-Objektes: gibt es die Datei?
		File theFile = new File( txtURL.getText() );
		if( ! theFile.exists() )
		{
			msg = new MsgBox( this, "Falsche URL", "Die Datei wurde nicht gefunden." );
			return null;
		}

		try
		{
			// ffnen der Datei
			DataInputStream dis = new DataInputStream( new FileInputStream( theFile ) );

			// jetzt wird der Text in die Variable bertragen
			while(dis.available() > 0)
				sBuffer += dis.readLine() + "\n";

		} catch( Exception e )
		{
			return null;
		}
*/

		DataList dataList;

		// ist es eine Reihe von Mewerten oder ein
		// Gleichungssystem
		if( isText( sBuffer ) )
		{
			// da der Inhalt des Textfiles in das Textfeld bertragen wird
			// wird es zuerst gelscht
			txtText.setText( sBuffer );

			// Jetzt den Text in Daten umwandeln
			dataList = parseText();
		}
		else
		{
			dataList = parseURLSystem( sBuffer );

			// konnte ein vollstndiges Gleichungssystem
			// gebildet werden
			if( dataList == null )
			{
				msg = new MsgBox( this, "Corrupt File", "The Content of the file could not be interpreted." );
				return null;
			}
			else
			{
   		// damit ich mitverfolgen kann,
		// ob das Applet arbeitet, bietet es die Reporter
		// Schnittstelle
dataList.setReporter( (Reporter)applet );


				// Daten generieren
				dataList.generateData();

				// nach dem Aufbauen einer DataList die Update-Methode
				// aufrufen: min, max, etc. Berechnung
				dataList.update();
			}
		}

		// Quelle Eintragen
		dataList.nOrigin = DataList.URL;

		// den URL fr spter verewigen
		dataList.sURL = txtURL.getText();

		return dataList;
	}


	DataList parseText()
	{
		DataList	dataList = new DataList(applet.go);
		//x StringTokenizer st = new StringTokenizer( txtText.getText(), "\n" );
        String sBuffer = txtText.getText();
		MsgBox msg;

		// damit ich mitverfolgen kann,
		// ob das Applet arbeitet, bietet es die Reporter
		// Schnittstelle
		dataList.setReporter( (Reporter)applet );


		if( isText( sBuffer ) )
		{
			// da der Inhalt des Textfiles in das Textfeld bertragen wird
			// wird es zuerst gelscht
			txtText.setText( sBuffer );

			// Jetzt den Text in Daten umwandeln
			dataList = parseText();
		}
		else
		{
			dataList = parseURLSystem( sBuffer );

			// konnte ein vollstndiges Gleichungssystem
			// gebildet werden
			if( dataList == null )
			{
				msg = new MsgBox( this, "Corrupt File", "The Content of the file could not be interpreted." );
				return null;
			}
			else
			{
				// Daten generieren
				dataList.generateData();

				// nach dem Aufbauen einer DataList die Update-Methode
				// aufrufen: min, max, etc. Berechnung
				dataList.update();
			}
		}


/*
		int dimension = 0;

		while( st.hasMoreElements()	)
		{
			DataElement data = new DataElement();
			
			StringTokenizer stl = new StringTokenizer( st.nextToken(), "," );
			while( stl.hasMoreElements() )
			{
				data.addElement( new Double( stl.nextToken() ) );
			}

			// soll verhindern, da Leerzeilen hinter den Daten
			// ebenfalls eingeparst werden
			if( dimension <= data.size() )
			{
				dataList.addElement( data );
				dimension = data.size();
			}
		}
*/
		// Quelle Eintragen
		dataList.nOrigin = DataList.TEXT;

		// nach dem Aufbauen einer DataList die Update-Methode
		// aufrufen: min, max, etc. Berechnung
//		dataList.update();

		return dataList;
	}

	Panel pnl;
	// Arbeitet die Ereignisse im Dialog ab

	public boolean action( Event evt, Object what )
	{
		if( evt.target instanceof Button )
		{
			String str = (String)what;

			if( str.equals(btnOk.getLabel()) )
			{
				// Dialog parsen
				DataList data;
                applet.requestFocus();

                if(applet.go != null)
                {
                  applet.go.setVisible(false);
                  applet.go.dispose();
                }

                applet.go = new GraphOutput(applet, "nDim-Viewer - Graphics Output", 640, 480);


				if( dataOrigin.getCurrent().getLabel().equals( "URL" ) )
				{
					data = parseURL();
				}
				else
				if( dataOrigin.getCurrent().getLabel().equals( "Text" ) )
				{
					data = parseText();
				}
				else
				{
					data = parseSystem();
				}

                applet.go.display();
                applet.go.setDynSysInfo(getDynSysInfo(data));

                setVisible(false);

				// konnte die Eingabe nicht analysiert werden,
				// kann der Dialog nicht mit OK verlassen werden
				if( data == null ) return true;

				((DataSink)applet).setData( data );
			    applet.setTime(0.0, (new Double(txtStep.getText())).doubleValue()*(new Double(txtIterations.getText())).doubleValue() );

				applet.repaint();

				//hide();
				dispose();
				return true;
			}
			if( str.equals(btnCancel.getLabel()) )
			{
				hide();
				dispose();
				return true;
			}
			if( str.equals(btnAdd.getLabel()) )
			{
				DataDialogAddDlg dlg = new DataDialogAddDlg( this );

				btnEdit.enable();
				btnDelete.enable();

				return true;
			}
			if( str.equals(btnEdit.getLabel()) )
			{
				int i = lstEquation.getSelectedIndex();

				if( i < 0 ) return true;

				DataDialogEditDlg dlg = new DataDialogEditDlg( this, i );

				return true;
			}
			if( str.equals(btnDelete.getLabel()) )
			{
				int i = lstEquation.getSelectedIndex();

				if( i < 0 ) return true;

				sBody.removeElementAt(findId(lstEquation.getSelectedItem()));
				elementOrigin.removeElementAt(findId(lstEquation.getSelectedItem()));
				sId.removeElementAt(findId(lstEquation.getSelectedItem()));
				
				lstEquation.delItem( i );
				if( lstEquation.countItems() < 1 )
				{
					btnEdit.disable();
					btnDelete.disable();
				}
				else
				{
					lstEquation.select( Math.min( i, lstEquation.countItems()-1 ) );
				}

				return true;
			}
			if( str.equals(btnBrowse.getLabel()) )
			{
				DataDialogBrowseDlg dlg = new DataDialogBrowseDlg( this );
				return true;
			}
			if( str.equals(btnHelp.getLabel()) )
			{
				HelpWindow hlp = new HelpWindow(applet, "help/data.hlp", 1, "Help - Data Dialog", 500, 400);
				hlp.show();
				return true;
			}
		}

		if( (evt.target instanceof Checkbox) &&
			( evt.id == Event.ACTION_EVENT ) )
		{
			if( dataOrigin.getCurrent().getLabel().equals( "URL" ) )
			{
				((CardLayout)pnl.getLayout()).show( pnl, "URL" );
			}
			else
			if( dataOrigin.getCurrent().getLabel().equals( "Equationsystem" ) )
			{
				((CardLayout)pnl.getLayout()).show( pnl, "System" );
			}
			else
			{
				((CardLayout)pnl.getLayout()).show( pnl, "Text" );
			}

			return true;
		}

		return false;
	}

	Button btnCancel;
	Button btnOk;
	Button btnHelp;
	
	Checkbox dataOriginSystem;
	Checkbox dataOriginURL;
	Checkbox dataOriginText;
	CheckboxGroup dataOrigin;

	// Darstellung der Eingabefelder
	TextField	txtURL;
	TextArea	txtText;

	// Button fr Auswahl eines vorgegebenen URLs
	Button btnBrowse;

	// Parent merken
//	Applet applet;
    NDimViewer applet;

	// Constructor

	public DataDialog( NDimViewer a )
	{
		super( "NDimViewer - Data" );

		applet = a;

		resize( 600, 400 );
		// setResizable( false );
		setLayout( new BorderLayout() );
		setBackground( Color.lightGray );

		//////////////////////////////////////////////////////
		// Hier werden die Eingabefelder des Dialogs aufgebaut

		// Ok und Abbrechen Schaltflche
		Panel p3 = new Panel();

		btnOk = new Button( "Ok" );
		btnCancel = new Button( "Cancel" );
        btnHelp = new Button("Help");
		
		p3.setLayout( new GridLayout(1,2, 13, 13) );
		p3.add( btnOk );
		p3.add( btnCancel );
		
		add( "South", p3 );

		// Optionsgruppe: Datenherkunft
		Panel p1 = new Panel();

		dataOrigin = new CheckboxGroup();

		dataOriginText = new Checkbox( "Text", dataOrigin, true );
		dataOriginURL = new Checkbox( "URL", dataOrigin, false );
		dataOriginSystem = new Checkbox( "Equationsystem", dataOrigin, false );

		p1.setLayout( new FlowLayout(FlowLayout.LEFT) );
		p1.add( new Label( "Origin:" ) );
		p1.add( dataOriginText );
		p1.add( dataOriginURL );
		p1.add( dataOriginSystem );
		p1.add( btnHelp );
		
		add( "North", p1 );

		pnl = new Panel();
		pnl.setLayout( new CardLayout() );

		////////////////////////////////
		// Eingabe fr Daten in Textfeld
		////////////////////////////////

		Panel pnlText = new Panel();
		txtText = new TextArea( 7, 30 );

		pnlText.setLayout( new GridLayout(1,1) );
		pnlText.add( txtText );


		pnl.add( "Text", pnlText );

		/////////////////////
		// Eingabe eines URLs
		/////////////////////

		Panel pnlURL = new Panel();
		txtURL = new TextField();
        txtURL.setEditable(false);
		btnBrowse = new Button( "Choose Example" );

		GridBagLayout layout = new GridBagLayout();
		GridBagConstraints cons = new GridBagConstraints();

		pnlURL.setLayout( layout );

		cons.anchor = 	GridBagConstraints.NORTHWEST;
		cons.insets = new Insets( 2, 2, 2, 2 );
		
		cons.fill = GridBagConstraints.NONE;
		cons.weightx = 0;
		pnlURL.add( btnBrowse );
		layout.setConstraints( btnBrowse, cons );

		Label lbl = new Label( "URL:" );
		pnlURL.add( lbl );
		layout.setConstraints( lbl, cons );

		cons.fill = GridBagConstraints.HORIZONTAL;
		cons.weightx = 1;
		pnlURL.add( txtURL );
		layout.setConstraints( txtURL, cons );

		pnl.add( "URL", pnlURL );



		///////////////////////////////////
		// Eingabe eines Gleichungssystems
		///////////////////////////////////

		Panel pnlSystem = new Panel();

		lstEquation = new java.awt.List();
		btnAdd = new Button( "Add..." );
		btnEdit = new Button( "Edit..." );
		btnDelete = new Button( "Delete" );

		btnEdit.disable();
		btnDelete.disable();


		layout = new GridBagLayout();
		cons = new GridBagConstraints();

		pnlSystem.setLayout( layout );

		cons.insets = new Insets( 2, 2, 2, 2 );
		cons.anchor = 	GridBagConstraints.WEST;
		
		lbl = new Label( "Equations:" );
		pnlSystem.add( lbl );
		layout.setConstraints( lbl, cons );

		cons.gridy = 1;
		cons.gridheight = 4;
		cons.gridwidth = 7;
		cons.fill = GridBagConstraints.BOTH;
		cons.weightx = 1;
		cons.weighty = 1;
		layout.setConstraints( lstEquation, cons );
		pnlSystem.add( lstEquation );

		cons.gridheight = 1;
		cons.gridwidth = 1;
		cons.gridx = 7;
		cons.weightx = 0;
		cons.weighty = 0;
		cons.fill = GridBagConstraints.HORIZONTAL;
		layout.setConstraints( btnAdd, cons );
		pnlSystem.add( btnAdd );


		cons.gridy = 2;
		layout.setConstraints( btnEdit, cons );
		pnlSystem.add( btnEdit );

		cons.gridy = 3;
		layout.setConstraints( btnDelete, cons );
		pnlSystem.add( btnDelete );

		// Auswahl des Nherungsverfahrens
		dataMethode = new CheckboxGroup();

		dataMethodeEuler = new Checkbox( "Euler", dataMethode, true );
		dataMethodeRungeKutta = new Checkbox( "Runge-Kutta", dataMethode, false );

		cons.gridx = 0;
		cons.gridy = 5;
		cons.fill = GridBagConstraints.NONE;
		cons.anchor = 	GridBagConstraints.WEST;

		lbl = new Label( "Methode:" );
		layout.setConstraints( lbl, cons );
		pnlSystem.add( lbl );

		cons.gridx = GridBagConstraints.RELATIVE;
		layout.setConstraints( dataMethodeEuler, cons );
		pnlSystem.add( dataMethodeEuler );

		layout.setConstraints( dataMethodeRungeKutta, cons );
		pnlSystem.add( dataMethodeRungeKutta );

		// Schrittweite fr die Berechnung
		lbl = new Label( "Stepwidth h:" );
		txtStep = new TextField("0.1",5);

		cons.gridx = 0;
		cons.gridy = 6;
		cons.fill = GridBagConstraints.NONE;
		// cons.anchor = 	GridBagConstraints.EAST;
		cons.anchor = 	GridBagConstraints.WEST;
		layout.setConstraints( lbl, cons );
		pnlSystem.add( lbl );
		
		cons.gridx = GridBagConstraints.RELATIVE;
		cons.anchor = 	GridBagConstraints.WEST;
		layout.setConstraints( txtStep, cons );
		pnlSystem.add( txtStep );

		// Iterationen
		lbl = new Label( "Iterations:" );
		txtIterations = new TextField("100",5);

		layout.setConstraints( lbl, cons );
		pnlSystem.add( lbl );

		layout.setConstraints( txtIterations, cons );
		pnlSystem.add( txtIterations );

		// Schrittweite zum Zeichnen
		lbl = new Label( "Display:" );
		txtDisplay = new TextField("10",5);

		layout.setConstraints( lbl, cons );
		pnlSystem.add( lbl );

		layout.setConstraints( txtDisplay, cons );
		pnlSystem.add( txtDisplay );

		lbl = new Label( "Points" );
		layout.setConstraints( lbl, cons );
		pnlSystem.add( lbl );
		
		
		pnl.add( "System", pnlSystem );

		add( "Center", pnl );
		((CardLayout)pnl.getLayout()).show( pnl, "Text" );

		/////////////////////////////////////////
		// bernahme bis jetzt eingegebener Daten
		/////////////////////////////////////////

		DataList data = ((DataSink)applet).getData();

		// wurden Daten bergeben? Dann mssen diese Dargestellt werden
		if( data != null )
		{
//(too slow)			initText( data );
			txtURL.setText( data.sURL );
			initSystem( data );

			// sind Gleichungen vorhanden, mssen die
			// Schaltflchen wieder angezeigt werden
			if( data.countEquation() + data.countHelper() > 0 )
			{
				btnEdit.enable();
				btnDelete.enable();
			}
		
			switch( data.nOrigin )
			{
			case DataList.URL:
						dataOrigin.setCurrent( dataOriginURL );
						((CardLayout)pnl.getLayout()).show( pnl, "URL" );
						break;
			case DataList.SYSTEM:
						dataOrigin.setCurrent( dataOriginSystem );
						((CardLayout)pnl.getLayout()).show( pnl, "System" );
						break;
			}					
		}
		
	}                          
	
	/**
	 * create an instance of DynSysInfo to send later to graphoutput
	 */
	public DynSysInfo getDynSysInfo(DataList data)
	{
	  DynSysInfo dsi = new DynSysInfo();

	  int i;
	  Vector sId = new Vector();
	  Vector sBody = new Vector();
	  Vector elementOrigin = new Vector();

	  for(i = 0; i < data.countEquation(); i++ )
	  {      
	    String s = new String(data.getName( i ) + "' = " + data.getEquation( i ) );
	    dsi.equation_v.addElement( s );
	  }

	  for(i = 0; i < data.countHelper(); i++ )
	  {
        String s = new String(data.getHelperId( i ) + " = " + data.getHelperBody( i ) );
        dsi.equation_v.addElement( s );
	  }

	  if( data.elementOrigin != null )
	  {
		elementOrigin = (DataElement)data.elementOrigin.clone();

		for( i = 0; i < elementOrigin.size(); i++ )
	  	  dsi.startvalue_v.addElement( new Double( ((Double)elementOrigin.elementAt(i)).doubleValue() ) );

	  }                
	                         
      if(data.nMethode == DataList.RUNGEKUTTA)
	    dsi.method = new String("Runge-Kutta");
	  else
	    dsi.method = new String("Euler");

	  dsi.stepwidth = data.nStep;
	  dsi.time = new String("0.0 to "+(data.nStep*(double)data.nIterations) );
	  dsi.disp_points = data.nDisplay;

      return dsi;
	}

} // end of class DataDialog




