package data;

import java.util.Vector;
import java.awt.*;
import expression.*;
import graphoutput.*;

/*
 *
 * DataList
 *
 */
public class DataList extends Vector implements Cloneable
{
    GraphOutput go;
	
    public DataList(GraphOutput go)
	{
	  super();
	  this.go=go;
	}
  
	public DataList clonedl()
	{
		DataList newData =(DataList) super.clone();

		int dim = dimension();

		newData.min = new double[ dim ];
		newData.max = new double[ dim ];

		for( int i = 0; i < dim; i++ )
		{
			newData.min[i] = min[i];
			newData.max[i] = max[i];
		}

		newData.elementOrigin = (DataElement)elementOrigin.clone();
		

		newData.nDisplay = nDisplay;
		newData.nIterations = nIterations;
		newData.nMethode = nMethode;
		newData.nOrigin = nOrigin;
		newData.nStep = nStep;
		newData.rptStatus = rptStatus;
		
		newData.sEquation = (Vector)sEquation.clone();
		newData.sHelper = (Vector)sHelper.clone();
		newData.sName = (Vector)sName.clone();
		newData.sURL = sURL;

		if( mapping != null )
			newData.mapping = (Vector)mapping.clone();

		return newData;
	}

	// Zugriffsfunktionen 

	public int countMapping()
	{
		// gibt die Anzahl der Achsen zurck. 
		// Die ist normalerweise gleich den Dimensionen,
		// kann aber auch hher sein, wenn z.B. eine
		// Dimension mehrfach verwendet wird
		if( mapping == null ) return dimension();

		return mapping.size();
	}

	public int getMapping( int axis )
	{
		// gibts kein Mapping, dann wird einfach die Achse
		// gleich der Dimension gesetzt 1=1, 2=2, etc.
		if( mapping == null ) return axis;

		// wurde nicht genug Zuordnungen gespeichert, 
		// wird ebenfalls nach dieser Regel vorgegangen
		if( axis >= mapping.size() ) return axis;

		// ansonsten hole den Wert aus dem Vector
		return ((Integer)mapping.elementAt(axis)).intValue();
	}

	public void setMapping( int axis, int dimension )
	{
		// gibt es bereits einen mapping-Vector
		if( mapping == null ) mapping = new Vector();

		// ist der Vector bereits gro genug
		for( int d = mapping.size(); d <= axis; d++ )
			mapping.addElement( new Integer(d) );

		// jetzt die gewnschte Dimension einsetzen
		mapping.setElementAt( new Integer(dimension), axis);	
	}

	public void clearMapping()
	{
		mapping = null;
	}

	// Speichert eine Zuordnung, welche "Achse" mit welcher 
	// Dimension belegt wird

	// Standardmig sollte die der Vektor einfach 1, 2, ... d
	// sein, wobei d die maximal verfgbaren Dimensionen sind

	protected Vector mapping;

	// Substituiert in die Gleichung sEquation die Gleichungen
	// hinein, die in sHelperVar bzw. sHelper untergebracht sind.

	// die Funktion ergibt die substituierte Gleichung zurck

	// Routine basiert auf einer Hilfsprozedur im dynsys-Applet
	// von R. Wegenkittel
	
	protected String Substitute( String sEquation, Vector sHelperVar, Vector sHelper ) 
	{
		// aus Performancegrnden
		if( sHelperVar.size() == 0 ) return sEquation;

		int top,akt,len,i;
		String output="", input="";
		char c;
		boolean isvar;

		// zuerst alle Leerzeichen aus den Gleichungen entfernen
		for(i=0;i<sEquation.length();i++)
			if (sEquation.charAt(i)!=' ')
				input = input + sEquation.charAt(i);

		len = input.length();
		top = akt = 0;

		c = input.charAt(akt);

		do
		{
			// bis zum nchsten Nicht-Operator lesen
			while((c!='(')&&(c!=')')&&(c!='+')&&(c!='-')&&(c!='*')&&(c!='/')&&(c!='^')&&(akt<len))
			{
				akt++;
				if (akt<len)
					c = input.charAt(akt);
			}

			if (top==akt)
			{
				output = output + c;
		
				akt++;
				top = akt;

				if (akt<len)
					c = input.charAt(akt);
			}
			else
			{
				// Kontrolle, ob das neu gefundene Token in der
				// Formel eine Variable zum Substituieren ist
				isvar = false;

				for(i=0;i<sHelperVar.size();i++)
				{
					String sName = (String)sHelperVar.elementAt(i);

					if (sName.equals(input.substring(top,akt)))
					{
						String sBody = (String)sHelper.elementAt(i);

						output = output + '(' + Substitute(sBody, sHelperVar, sHelper) + ')';
						isvar = true;
					}
				}

				// ist der Token keine Variable, wird der Token
				// in der Formel belassen (eventuell eine Funktion)
				if (!isvar)
					output = output + input.substring(top,akt);
		
				top = akt;
			}
		}while(akt<len);

		return output;
	}

	// Zugriffsfunktionen fr dieses Feld
	public void setReporter( Reporter r ) { rptStatus = r; }
	public Reporter getReporter() { return rptStatus; }

	// dieses Feld kann dazu verwendet werden, 
	// den Status der Berechnung sichtbar zu machen
	protected Reporter rptStatus;
	
	// Zugriffsfunktionen fr das Array mit den 
	// Hilfsgleichungen
	String getHelperBody( int index )
	{
		return (String)sHelper.elementAt( index );
	}

	String getHelperId( int index )
	{
		return (String)sHelperVar.elementAt( index );
	}

	// Gibt den Gleichungskrper fr die index-te
	// Gleichung zurck

	String getEquation( int index )
	{
		return (String)sEquation.elementAt( index );
	}

	// wieviele Gleichungen habe ich gespeichert

	int countEquation() 
	{
		return sEquation.size();
	}

	// wieviele Gleichungen habe ich gespeichert

	int countHelper() 
	{
		return sHelper.size();
	}

	// Lscht alle bereits bergebenen Hilfsgleichungen
	// aus dem Datenelement

	void clearEquation() 
	{
		sEquation = new Vector();
		sName = new Vector();
	}

	// Fgt eine Gleichung dem Gleichungssystem hinzu

	void addEquation( String sId, String sBody )
	{
		// Fgt den Namen ein (aber ohne ' von x')
		sName.addElement( sId.substring( 0,1 ) );

		// und den Funktionskrper
		sEquation.addElement( sBody );
	}

	// Lscht alle Hilfsgleichungen

	void clearHelper()
	{
		sHelperVar = new Vector();
		sHelper = new Vector();
	}

	// Fgt eine Hilfsgleichung dem System hinzu

	void addHelper( String sId, String sBody )
	{
		sHelper.addElement( sBody );
		sHelperVar.addElement( sId );
	}

	// Verndert eine Hilfsgleichung.

	// Wird verwendet, um interaktiv einen Parameter zu ndern

	void editHelper( int nIndex, String sBody )
	{
		sHelper.setElementAt( sBody, nIndex );
	}

	// Subprozedur, die die Generierung der Daten mittels
	// Euler betreibt

	boolean generateDataEuler( Vector sNewEquation, long nDisplayStep )
	{
		int d;
		long i;

		// wir starten beim benutzerdefinierten Startpunkt
		DataElement x = (DataElement)elementOrigin.clone();

		for( i = 0; i < nIterations; i++ )
		{
			if( rptStatus != null )
				rptStatus.setStatus( "Calculation " + Long.toString( i * 100 /  nIterations ) + "%" );

			// den aktuellen Punkt tragen wir ein
			if( i % nDisplayStep == 0 )
				// ja, wir haben wieder einen Punkt erreicht,
				// an dem wir ein Element eintragen knnen
			{
				addElement( x.clone() );
				go.addPoint( x.getDoubleVector() );
            }
			
			// bertrage den aktuellen Punkt in die
			// globalen Variablen des Formelsystems
			x.copyIntoVariables( sName );
			// und jetzt rechnen wir den nchsten Schritt
			// und zwar fr jede Dimension
	        
			for( d = 0; d < sName.size(); d++ )
			{
				Expression expX;
				double x1, x2;
		
				try
				{
					// das ist der Punkt auf dem ich stehe
					x1 = ((Double)x.elementAt(d)).doubleValue();

					// das ist der Ausdruck zur Berechnung von x'

					expX = new Expression( (String)sNewEquation.elementAt(d) );

					// dann ergibt sich x2 = h * x' + x1
					x2 = nStep * expX.eval() + x1;

					// dann trage diese Komponente an der
					// Stelle d wieder in den Vektor ein
					x.setElementAt( new Double(x2), d );
				}
				catch( Exception e )
				{
					if( rptStatus != null )
						rptStatus.setStatus(e.getMessage());
					removeAllElements();
					return false;
				}
				
			}
		}

		if( rptStatus != null )
			rptStatus.setStatus( "" );

		return true;
	}

	// Subprozedur, die die Generierung der Daten mittels
	// Runge-Kutta betreibt

	boolean generateDataRungeKutta( Vector sNewEquation, long nDisplayStep )
	{
		int d;
		long i;

		int dim = sName.size();

		// wir starten beim benutzerdefinierten Startpunkt
		DataElement x = (DataElement)elementOrigin.clone();

		// in dieser Variable wird der Prognosewert der
		// vier Zwischenschritte errechnet
		DataElement z = (DataElement)elementOrigin.clone();

		for( i = 0; i < nIterations; i++ )
		{
			if( rptStatus != null )
				rptStatus.setStatus( "Calculation " + Long.toString( i * 100 /  nIterations ) + "%" );

			// den aktuellen Punkt tragen wir ein
			if( i % nDisplayStep == 0 )
				// ja, wir haben wieder einen Punkt erreicht,
				// an dem wir ein Element eintragen knnen
			{	
				addElement( x.clone() );
			    go.addPoint( x.getDoubleVector() );
			}

			// bertrage den aktuellen Punkt in die
			// globalen Variablen des Formelsystems
			x.copyIntoVariables( sName );

			// Temporre Arrays fr Hilfssteigungen
			// fr jede Dimension wird eine eigene errechnet
			double[] k1 = new double[dim]; 
			double[] k2 = new double[dim]; 
			double[] k3 = new double[dim]; 
			double[] k4 = new double[dim]; 

			// Errechnung des ersten Hilfsschrittes
			// und zwar fr jede Dimension
						
			for( d = 0; d < dim; d++ )
			{
				Expression expX;
				double x1, x2;
	
				try
				{
					// das ist der Ausdruck zur Berechnung von x'
					// d.h. fr die 1. Hilfssteigung
					expX = new Expression( (String)sNewEquation.elementAt(d) );
					k1[d] = expX.eval();

					// das ist der Punkt auf dem ich stehe
					x1 = ((Double)x.elementAt(d)).doubleValue();

					// dann ergibt sich lt. Runge-Kutta der 
					// 1. Prognosehilfswert als x2 = h/2 * k1 + x1
					x2 = nStep/2 * k1[d] + x1;

					// dann trage diese Komponente an der
					// Stelle d in den Vektor z ein
					z.setElementAt( new Double(x2), d );
				}
				catch( Exception e )
				{
					if( rptStatus != null )
						rptStatus.setStatus(e.getMessage());
					removeAllElements();
					return false;
				}
			}

			// damit habe ich jetzt k1 und in x steht der 
			// 1. Prognosehilfswert. Diesen setze ich jetzt
			// wieder ins Gleichungssystem ein

			z.copyIntoVariables( sName );

			// Errechnung des zweiten Hilfsschrittes
			// und zwar fr jede Dimension
			for( d = 0; d < dim; d++ )
			{
				Expression expX;
				double x1, x2;
	
				try
				{
					// das ist der Ausdruck zur Berechnung von x'
					// d.h. fr die 2. Hilfssteigung
					expX = new Expression( (String)sNewEquation.elementAt(d) );
					k2[d] = expX.eval();

					// das ist der Punkt auf dem ich stehe
					x1 = ((Double)x.elementAt(d)).doubleValue();

					// dann ergibt sich lt. Runge-Kutta der 
					// 2. Prognosehilfswert als x2 = h/2 * k2 + x1
					x2 = nStep/2 * k2[d] + x1;

					// dann trage diese Komponente an der
					// Stelle d in den Vektor z ein
					z.setElementAt( new Double(x2), d );
				}
				catch( Exception e )
				{
					if( rptStatus != null )
						rptStatus.setStatus(e.getMessage());
					removeAllElements();
					return false;
				}
			}

			// damit habe ich jetzt k1, k2 und in x steht der 
			// 2. Prognosehilfswert. Diesen setze ich jetzt
			// wieder ins Gleichungssystem ein

			z.copyIntoVariables( sName );

			// Errechnung des dritten Hilfsschrittes
			// und zwar fr jede Dimension
			for( d = 0; d < dim; d++ )
			{
				Expression expX;
				double x1, x2;
	
				try
				{
					// das ist der Ausdruck zur Berechnung von x'
					// d.h. fr die 3. Hilfssteigung
					expX = new Expression( (String)sNewEquation.elementAt(d) );
					k3[d] = expX.eval();

					// das ist der Punkt auf dem ich stehe
					x1 = ((Double)x.elementAt(d)).doubleValue();

					// dann ergibt sich lt. Runge-Kutta der 
					// 3. Prognosehilfswert als x2 = h/2 * k3 + x1
					x2 = nStep * k3[d] + x1;

					// dann trage diese Komponente an der
					// Stelle d wieder in den Vektor ein
					z.setElementAt( new Double(x2), d );
				}
				catch( Exception e )
				{
					if( rptStatus != null )
						rptStatus.setStatus(e.getMessage());
					removeAllElements();
					return false;
				}
			}

			// damit habe ich jetzt k1, k2, k3 und in x steht der 
			// 3. Prognosehilfswert. Diesen setze ich jetzt
			// wieder ins Gleichungssystem ein

			z.copyIntoVariables( sName );

			// Errechnung des vierten Hilfsschrittes
			// und der groen Gesamtsteigung
			// und zwar fr jede Dimension
			for( d = 0; d < dim; d++ )
			{
				Expression expX;
				double x1, x2;
	
				try
				{
					// das ist der Ausdruck zur Berechnung von x'
					// d.h. fr die 4. Hilfssteigung
					expX = new Expression( (String)sNewEquation.elementAt(d) );
					k4[d] = expX.eval();

					// das ist der Punkt auf dem ich stehe
					x1 = ((Double)x.elementAt(d)).doubleValue();

					// jetzt aus den vier Hilfssteigungen die
					// echte endgltige errechnen
					double h = (k1[d]+2*k2[d]+2*k3[d]+k4[d])/6;

					// dann ergibt sich lt. Runge-Kutta der 
					// 2. Prognosehilfswert als x2 = h/2 * k3 + x1
					x2 = nStep * h + x1;

					// dann trage diese Komponente an der
					// Stelle d wieder in den Vektor ein
					x.setElementAt( new Double(x2), d );
				}
				catch( Exception e )
				{
					if( rptStatus != null )
						rptStatus.setStatus(e.getMessage());
					removeAllElements();
					return false;
				}
			}

		}

		if( rptStatus != null )
			rptStatus.setStatus( "" );

		return true;
	}


	// generiert aus dem Gleichungssystem und den
	// Iterationswerten Daten
	boolean generateData()
	{
		// zuerst alle Werte entfernen
		removeAllElements();

		// In die Gleichungen mssen noch die Hilfsgleichungen
		// eingesetzt werden. Dazu wird ein neues Array aufgebaut,
		// das sEquation entspricht, in das sHelper hinein-
		// substitutiert wurde
		Vector	sNewEquation = new Vector();// Gleichung fr eine Dimension

		for( int d = 0; d < sName.size(); d++ )
		{
			// wie hat die Gleichung bis jetzt ausgesehen
			String sTemp = new String((String)sEquation.elementAt(d));

			// und jetzt die Hilfsgleichungen hineinsubstituieren
			sTemp = Substitute( sTemp, sHelperVar, sHelper );

			// und in das neue Gleichungssystem integrieren
			sNewEquation.addElement( sTemp );
		}

		// es sollen nur nDisplay Punkte dargestellt werden.
		// Welcher Schrittweite entspricht das
		long nDisplayStep = 1;
		
		if( nDisplay > 1 )
			nDisplayStep = (nIterations - 1) / (nDisplay - 1);

        // init the graphoutput-window for receiving the data from this
		go.initOutput(sName.size(), sName, sName, new Range(0.0, nStep*(double)nIterations) );

		// jetzt entweder mit Euler oder mit Runge-Kutta rechnen
		boolean result = true;
		if( nMethode == EULER )
			result = generateDataEuler(sNewEquation, nDisplayStep);
		else
			result = generateDataRungeKutta(sNewEquation, nDisplayStep);

//		go.display();
		return result;
	}

	// das Gleichungssystem

	DataElement	elementOrigin;		    // enthlt den Startpunkt fr die Iteration
	public double	nStep = 0.1;		// Schrittweite
	public long		nIterations = 100;	// Anzahl der Iterationen
	public long		nDisplay = 10;		// Wieviele werden davon angezeigt

	public int		nMethode = 0;		// Methode: Euler oder Runge-Kutta

	public static final int EULER = 0;
	public static final int RUNGEKUTTA = 1;

	Vector		sEquation = new Vector();// Gleichung fr eine Dimension
	
	Vector		sHelperVar = new Vector();	// Hilfsvariablen
	Vector		sHelper = new Vector();		// Hilfsausdrcke

	// Zugriffsfunktionen fr die Benennung der
	// Dimensionen

	// Zhlung fr die Dimension beginnt bei 0

	public String getName( int dimension )
	{
		// Unsinn, wird mit Unsinn belohnt
		if( dimension < 0 ) return "";

		// fr diese Dimension habe ich keinen
		// Namen also wird einer generiert
		if( dimension >= sName.size() ) 
			return "X" + Integer.toString( dimension + 1 );

		// ansonsten hole den Namen aus dem Array
		return (String)sName.elementAt( dimension );
	}

	public void setName( int dimension, String sNewName )
	{
		// Unsinnige Dimensionen werden nicht beachtet
		if( dimension < 0 ) return;

		// ist das Array bereits gro genug
		if( dimension >= sName.size() )
		{
			int d;

			// Nein, also vergrern und
			// vorinitialisieren
			for( d = sName.size(); d <= dimension; d++ )
				sName.addElement( "X" + Integer.toString( d + 1 ) );
		}

		// den bergebenen Namen speichern
		sName.setElementAt(sNewName, dimension);
	}

	// Enthlt fr jede Dimension einen Namen

	protected Vector sName = new Vector();

	// Gibt alle Daten als Zeichenkette zurck, die einzelnen
	// Mewerte durch "\n" getrennt

	String getText() 
	{
		String s = new String();

		for( int i = 0; i < size(); i++ )
		{
			s = s + ((DataElement)elementAt(i)).getText();
			if( i + 1 < size() ) s = s + "\n";
		}

		// return toString();
		return s;
	}

	// Enthlt eine Kennung, woher die Daten kommen (Texteingabe, URL, Gleichungssystem)
	int nOrigin;

	static final int TEXT = 0;
	static final int URL = 1;
	static final int SYSTEM = 2;

	// Enthlt den URL oder das Gleichungssystem, aus dem die Daten errechnet wurden
	String sURL;

	double min[];
	double max[];

	public void paint( Graphics g, Rectangle r ) 
	{
		// Zeichnen der Parallelen Koordinaten-Achsen
		int parts = dimension()-1;

		for( int i = 1; i < parts; i++ )
		{
			g.drawLine( r.x + i * (r.width / parts),
						r.y,
						r.x + i * (r.width / parts),
						r.y + r.height );
		}

		for( int i = 0; i < size(); i++ )
		{
			g.setColor( chooseColor(i) );

			((DataElement)elementAt(i)).paint( g, r, min, max );
		}
	}

	Color chooseColor( int i )
	{
		switch(i % 8)
		{
		case 0:
			return Color.blue;
		case 1:
			return Color.cyan;
		case 2:
			return Color.green;
		case 3:
			return Color.magenta;
		case 4:
			return Color.orange;
		case 5:
			return Color.pink;
		case 6:
			return Color.red;
		case 7:
			return Color.white;
		}

		// never executed
		return Color.black;
	}

	// Gibt die Dimension der gespeicherten Daten zurck

	public int dimension()
	{
		if( size() > 0 )
			return ((DataElement)elementAt(0)).size();
		else
			return 0;
	}

	// Kopiert die Minima, Maxima aus dem 
	// bergebenen Datenmodell in dieses

	public void copyMinMax( DataList data )
	{
		int dim = dimension();

		// sind die Dimensionen ungleich, macht
		// eine bertragung keinen Sinn
		if( dim != data.dimension() ) return;

		min = new double[ dim ];
		max = new double[ dim ];

		for( int i = 0; i < dim; i++ )
		{
			min[i] = data.min[i];
			max[i] = data.max[i];
		}
	}

	// Nach dem Aufbau einer neuen DataList durch den
	// DataDialog werden die internen Flags korrigiert

	// Hauptschlich werden die Minima/Maxima fr die
	// sptere Skalierung des Modells neu berechnet

	public void update()
	{
		int dim = dimension();

		// Min, Max - Berechnung

		min = new double[ dim ];
		max = new double[ dim ];

		for( int d = 0; d < dim; d++ )
			min[d] = max[d] = ((Double)((DataElement)elementAt(0)).elementAt(d)).doubleValue();

		for( int i = 1; i < size(); i++ )
			for( int d = 0; d < dim; d++ )
			{
				double x = ((Double)((DataElement)elementAt(i)).elementAt(d)).doubleValue();

				if( min[d] > x ) min[d] = x;
				if( max[d] < x ) max[d] = x;
			}


		// Kontrolllauf, ob fr eine der Dimensionen
		// min und max gleich gro. Das wrde beim
		// Skalieren mit (x-min)/(max-min) eine Division
		// durch 0 auslsen.
		//if( rptStatus != null )
		//	rptStatus.setStatus( "Min/Max Korrektur beginnt" );

		for( int d = 0; d < dim; d++ )
		{
			// if( rptStatus != null )
			//	rptStatus.setStatus( Double.toString(min[d]) + "/" + Double.toString(max[d]) );

			if( min[d] == max[d] )
			{
				// Willkr:
				// min wird um eins erniedrigt,
				// max wird um eins erhht
				min[d] -= 1.0;
				max[d] += 1.0;

				if( rptStatus != null )
					rptStatus.setStatus( "Min/Max Correction at dimension no. " + Integer.toString(d+1) );
			}
		}
	}
}

