/*    $Id: FunctionSet.java,v 1.3 1996/03/22 03:28:30 davidliu Exp $
**
**
**************************************************************************
**    Copyright (C) 1996 David Wanqian Liu
**                       davidliu@mti.sgi.com
**                       http://reality.sgi.com/exmployees/davidliu_mti
**
**    This program is free software; you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation; either version 2 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program; if not, write to the Free Software
**    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
**************************************************************************
*/


package expression;



import java.util.Hashtable;



/**
  The set of all available functions.
  */
public final class FunctionSet {
  /**
    Function a+b.
    */
  final static Function add = new addFunc();

  /**
    Function a-b.
    */
  final static Function sub = new subFunc();

  /**
    Function a*b.
    */
  final static Function mult = new multFunc();

  /**
    Function a/b.
    */
  final static Function div = new divFunc();

  /**
    Function a^b.
    */
  final static Function pow = new powFunc();

  /**
    Function abs(a).
    */
  final static Function abs = new absFunc();

  /**
    Function acos(a).
    */
  final static Function acos = new acosFunc();

  /**
    Function actan(a).
    */
  final static Function actan = new actanFunc();

  /**
    Function asin(a).
    */
  final static Function asin = new asinFunc();

  /**
    Function atan(a).
    */
  final static Function atan = new atanFunc();

  /**
    Function box(a).
    */
  final static Function box = new boxFunc();

  /**
    Function ceil(a).
    */
  final static Function ceil = new ceilFunc();

  /**
    Function cos(a).
    */
  final static Function cos = new cosFunc();

  /**
    Function ctan(a).
    */
  final static Function ctan = new ctanFunc();

  /**
    Function exp(a).
    */
  final static Function exp = new expFunc();

  /**
    Function floor(a).
    */
  final static Function floor = new floorFunc();

  /**
    Function log(a).
    */
  final static Function log = new logFunc();

  /**
    Function round(a).
    */
  final static Function round = new roundFunc();

  /**
    Function sin(a).
    */
  final static Function sin = new sinFunc();

  /**
    Function sinc(a).
    */
  final static Function sinc = new sincFunc();

  /**
    Function sqrt(a).
    */
  final static Function sqrt = new sqrtFunc();

  /**
    Function sqr(a).
    */
  final static Function sqr = new sqrFunc();

  /**
    Function tan(a).
    */
  final static Function tan = new tanFunc();

  /**
    Hash table to store all available functions.
    */
  static Hashtable funcTab = null;

  static {
    funcTab = new Hashtable(20*2);

    funcTab.put("+", add);
    funcTab.put("ADD", add);
    funcTab.put("add", add);

    funcTab.put("-", sub);
    funcTab.put("SUB", sub);
    funcTab.put("sub", sub);

    funcTab.put("*", mult);
    funcTab.put("MULT", mult);
    funcTab.put("mult", mult);

    funcTab.put("/", div);
    funcTab.put("DIV", div);
    funcTab.put("div", div);

    funcTab.put("^", pow);
    funcTab.put("POW", pow);
    funcTab.put("pow", pow);

    funcTab.put("ABS", abs);
    funcTab.put("abs", abs);

    funcTab.put("ACOS", acos);
    funcTab.put("acos", acos);

    funcTab.put("ASIN", asin);
    funcTab.put("asin", asin);

    funcTab.put("ACTAN", actan);
    funcTab.put("actan", actan);

    funcTab.put("ATAN", atan);
    funcTab.put("atan", atan);

    funcTab.put("BOX", box);
    funcTab.put("box", box);

    funcTab.put("CEIL", ceil);
    funcTab.put("ceil", ceil);

    funcTab.put("COS", cos);
    funcTab.put("cos", cos);

    funcTab.put("EXP", exp);
    funcTab.put("exp", exp);

    funcTab.put("EXP10", exp);
    funcTab.put("exp10", exp);

    funcTab.put("FLOOR", floor);
    funcTab.put("floor", floor);

    funcTab.put("LOG", log);
    funcTab.put("log", log);

    funcTab.put("LN", log);
    funcTab.put("ln", log);

    funcTab.put("ROUND", round);
    funcTab.put("round", round);

    funcTab.put("SIN", sin);
    funcTab.put("sin", sin);

    funcTab.put("SINC", sinc);
    funcTab.put("sinc", sinc);

    funcTab.put("SQR", sqr);
    funcTab.put("sqr", sqr);

    funcTab.put("SQRT", sqrt);
    funcTab.put("sqrt", sqrt);

    funcTab.put("TAN", tan);
    funcTab.put("tan", tan);

    funcTab.put("CTAN", ctan);
    funcTab.put("ctan", ctan);
  }


  /**
    Fetch a function by passing in its string representation.
    */
  public static Function getFunc(String fstr) {
    return (Function) funcTab.get(fstr);
  }

}
