package graphoutput;

import java.awt.*;
import java.awt.event.*;
import java.applet.*;      
import javax.media.j3d.*;
import javax.vecmath.*;

/**
 *  Dialog to specify the attributes of a CuttingPlane
 */
class CuttingPlaneDlg extends Dialog
implements WindowListener, ActionListener, ItemListener
{
  Frame parent;
  CuttingPlane cp;

  CardLayout cl;
  Panel p22;

  CheckboxGroup cbg;
  Button col;
  TextField trans;
  TextField pos;
  TextField px, py, pz, vx, vy, vz;

  public CuttingPlaneDlg(Frame parent, CuttingPlane cp)
  {
	super( parent, "Cutting Plane Dialog", true );
    this.parent = parent;
    this.cp = cp;

    createComponents();

    pack();
    addWindowListener(this);
    setVisible(true);
  }


  public void actionPerformed(ActionEvent e)
  {
	if(e.getActionCommand().equals("Change"))
	{
      new ColorDlg(this, (Button)e.getSource());
	}
	if(e.getActionCommand().equals("Cancel"))
	{
      setVisible(false);
      dispose();
	}
	if(e.getActionCommand().equals("OK"))
	{
	  updateCP();
      setVisible(false);
      dispose();
	}
  }

   /**
    * Is called if the state of the Checkbox is changed
	*/
   public void itemStateChanged(ItemEvent e)
   {
     if(((String)e.getItem())=="Freelance")
       cl.show(p22, "freelance");
     else
       cl.show(p22, "rectangular");
   }


  // writes the information from the components into
  // this.cp
  private void updateCP()
  {
    Color c = col.getBackground();
    Color3f c3f = new Color3f( ((float)c.getRed())/255.0f,
                               ((float)c.getGreen())/255.0f,
                               ((float)c.getBlue())/255.0f );

    cp.setColor(c3f, (new Float(trans.getText())).floatValue());

    double frac = (new Integer(pos.getText())).doubleValue()/100.0;

    if(cbg.getSelectedCheckbox().getLabel().equals("Freelance"))
    {
      Point3d p = new Point3d((new Double(px.getText())).doubleValue(),
                              (new Double(py.getText())).doubleValue(),
                              (new Double(pz.getText())).doubleValue() );
      Vector3d v = new Vector3d((new Double(vx.getText())).doubleValue(),
                                (new Double(vy.getText())).doubleValue(),
                                (new Double(vz.getText())).doubleValue() );

      cp.setPosition(p, v);
      cp.generateAutoDescription();
    }
    else
    if(cbg.getSelectedCheckbox().getLabel().equals("XY-Plane"))
    {
      cp.setPosition(new Point3d(0.5, 0.5, frac),
                     new Vector3d(0.0, 0.0, 1.0) );
      cp.setDescription("XY-Plane at "+ (new Integer(pos.getText())) + "%");
    }
    if(cbg.getSelectedCheckbox().getLabel().equals("YZ-Plane"))
    {
      cp.setPosition(new Point3d(frac, 0.5, 0.5),
                     new Vector3d(1.0, 0.0, 0.0) );
      cp.setDescription("YZ-Plane at "+ (new Integer(pos.getText())) + "%");
    }
    if(cbg.getSelectedCheckbox().getLabel().equals("XZ-Plane"))
    {
      cp.setPosition(new Point3d(0.5, frac, 0.5),
                     new Vector3d(0.0, 1.0, 0.0) );
      cp.setDescription("XZ-Plane at "+ (new Integer(pos.getText())) + "%");
    }


    ((OptionsDlg)parent).updateList();

  }

  private void createComponents()
  {
    setBackground(Color.white);
    setLayout(new BorderLayout());

    Panel p1 = new Panel();
    p1.add(new Label("Color:", Label.RIGHT));
    col = new Button("Change");
    col.addActionListener(this);
    Color3f c1 = cp.getColor();
    col.setBackground(new Color(c1.x, c1.y, c1.z));
    p1.add(col);
    p1.add(new Label("Transparency:", Label.RIGHT));
    trans = new TextField(""+cp.getTransparency());
    p1.add(trans);
    add("North", p1);

    Panel p2 = new Panel();
    GridBagLayout gbl = new GridBagLayout();
    p2.setLayout(gbl);
    GridBagConstraints gbc = new GridBagConstraints();
    Panel p21 = new Panel();
    p21.setLayout(new GridLayout(4,1));
    cbg = new CheckboxGroup();
    Checkbox cb1 = new Checkbox("XY-Plane", false, cbg);
    Checkbox cb2 = new Checkbox("XZ-Plane", false, cbg);
    Checkbox cb3 = new Checkbox("YZ-Plane", false, cbg);
    Checkbox cb4 = new Checkbox("Freelance", false, cbg);
    int position = 50;
    Point3d p3d = cp.getPoint();
    String card = new String("rectangular");
    Vector3d v3d = cp.getNormalVector();
    if(cp.description.startsWith("XY")) {
      cb1.setState(true);
      position = (int)(100.0*p3d.z);   }
    if(cp.description.startsWith("XZ")) {
      cb2.setState(true);
      position = (int)(100.0*p3d.y);   }
    if(cp.description.startsWith("YZ")) {
      cb3.setState(true);
      position = (int)(100.0*p3d.x);   }
    if(cp.description.startsWith("p=")) {
       cb4.setState(true);
       card = new String("freelance"); }
    cb1.addItemListener(this);
    cb2.addItemListener(this);
    cb3.addItemListener(this);
    cb4.addItemListener(this);
    p21.add(cb1);
    p21.add(cb2);
    p21.add(cb3);
    p21.add(cb4);
    gbc.gridx=0;
    gbc.gridy=0;
    gbc.gridwidth=1;
    gbc.gridheight=1;
    gbl.setConstraints(p21, gbc);
    p2.add(p21);

    p22 = new Panel();
    cl = new CardLayout();
    p22.setLayout(cl);

    Panel p221 = new Panel();
    p221.add(new Label("At Position ", Label.RIGHT));
    pos = new TextField(""+position, 3);
    p221.add(pos);
    p221.add(new Label("%", Label.LEFT));
 //   p22.add("North", p221);
    p22.add("rectangular", p221);

    Panel p222 = new Panel();
    p222.setLayout(new GridLayout(3,1));
    p222.add(new Label("1x1x1 - Cube:", Label.CENTER));
    Panel p2221 = new Panel();
    p2221.add(new Label("p=(", Label.RIGHT));
    px = new TextField(""+p3d.x,5);
    p2221.add(px);
    p2221.add(new Label(", ", Label.CENTER));
    py = new TextField(""+p3d.y,5);
    p2221.add(py);
    p2221.add(new Label(", ", Label.CENTER));
    pz = new TextField(""+p3d.z,5);
    p2221.add(pz);
    p2221.add(new Label(")", Label.LEFT));
    p222.add(p2221);


    Panel p2222 = new Panel();
    p2222.add(new Label("v=(", Label.RIGHT));
    vx = new TextField(""+v3d.x,5);
    p2222.add(vx);
    p2222.add(new Label(", ", Label.CENTER));
    vy = new TextField(""+v3d.y,5);
    p2222.add(vy);
    p2222.add(new Label(", ", Label.CENTER));
    vz = new TextField(""+v3d.z,5);
    p2222.add(vz);
    p2222.add(new Label(")", Label.LEFT));
    p222.add(p2222);
    p22.add("freelance", p222);

    GridBagConstraints gbc1 = new GridBagConstraints();
    gbc1.gridx=1;
    gbc1.gridy=0;
    gbc1.gridwidth=2;
    gbc1.gridheight=1;
    gbl.setConstraints(p22, gbc1);
    p2.add(p22);


    add("Center", p2);


    Panel p3 = new Panel();
    Button b1 = new Button("OK");
    b1.addActionListener(this);
    p3.add(b1);
    Button b2 = new Button("Cancel");
    b2.addActionListener(this);
    p3.add(b2);
    add("South", p3);

    cl.show(p22, card);

    pack();

  }




    public void windowActivated(WindowEvent e) {}
    public void windowClosed(WindowEvent e) {}
    public void windowClosing(WindowEvent e)
	{
      setVisible(false);
	  dispose();
	}
    public void windowDeactivated(WindowEvent e) {}
    public void windowDeiconified(WindowEvent e) {}
    public void windowIconified(WindowEvent e) {}
    public void windowOpened(WindowEvent e) {}


}

