package graphoutput;

import java.awt.*;
import java.awt.event.*;
import java.applet.*;   
import java.util.*;     
import javax.vecmath.Color3f;
import javax.media.j3d.*;

/**
 * OptionsDlg - to change attributes of the existing graph
 * divided into sections:
 *    scaling, colors, cutting-planes, mapping, animation,
 *    highlighting
 */
class OptionsDlg extends Frame
implements WindowListener, ActionListener, ItemListener
{               
    GraphOutput go;

    CardLayout cl;
    Panel clp;         
    // scaling
    Vector sc_min_v = new Vector();
    Vector sc_max_v = new Vector();
    TextField sc_x, sc_y, sc_z;
    Vector co_col = new Vector();
    Checkbox co_showgrid;
    Checkbox co_showlines;
    Checkbox co_showplanes;
    Checkbox co_showsurface;
    TextField co_transparency;
    TextField co_fraction;
    Vector cp_planes;     
    java.awt.List cp_list; 
    java.awt.List ma_list1, ma_list2;  
    java.awt.List hl_list;
    Vector hl_markers;

	public OptionsDlg( GraphOutput go )
	{
		super( "Options" );

		this.go = go;

        // first, create the checkboxgroup
        Panel p = new Panel();                                           
        p.setLayout(new FlowLayout());
        CheckboxGroup cbg = new CheckboxGroup();
        Checkbox scaling = new Checkbox("Scaling", cbg, true);
        Checkbox colors = new Checkbox("Colors", cbg, false);
        Checkbox cutting_planes = new Checkbox("Cutting-Planes", cbg, false);
        Checkbox mapping = new Checkbox("Mapping", cbg, false);
        Checkbox animation = new Checkbox("Animation", cbg, false);
        Checkbox highlighting = new Checkbox("Highlighting", cbg, false);
        scaling.addItemListener(this);
        colors.addItemListener(this);
        cutting_planes.addItemListener(this);
        mapping.addItemListener(this);
        animation.addItemListener(this);
        highlighting.addItemListener(this);
        scaling.setBackground(Color.lightGray);
        colors.setBackground(Color.lightGray);
        cutting_planes.setBackground(Color.lightGray);
        mapping.setBackground(Color.lightGray);
        animation.setBackground(Color.lightGray);
        highlighting.setBackground(Color.lightGray);
        p.add(scaling);
        p.add(colors);
        p.add(cutting_planes);
        p.add(mapping);
// so far        p.add(animation);
        p.add(highlighting); 
        p.setBackground(Color.lightGray);

        // second, the bottom - button - line
        Panel p1 = new Panel();
        p1.setLayout(new FlowLayout(FlowLayout.RIGHT)); 
        Button apply = new Button("Apply");
        Button cancel = new Button("Cancel");
        Button ok = new Button("OK");
        apply.addActionListener(this);
        cancel.addActionListener(this);
        ok.addActionListener(this);
//        p1.add(apply);
        p1.add(cancel);
        p1.add(ok);
        
        Panel cl = createCardPanel();

        setLayout(new BorderLayout());
        add("North", p);    
        add("Center", cl);
        add("South", p1);           

        pack();
		addWindowListener(this);
		
	}

  	/**
	 *  eventhandling
	 */
	public void actionPerformed(ActionEvent e)
    {
	  if(e.getActionCommand().equals("Cancel"))
	  { 
	    go.optionsdlg=null;
	    setVisible(false);
	    dispose();
	  }
	  else if(e.getActionCommand().equals("OK"))
	  {
	    updateGraphoutput();
	    go.optionsdlg=null;
	    setVisible(false);
	    dispose();
	  }
	  else if(e.getActionCommand().equals("Apply"))
	  {
	    updateGraphoutput();
	  }
	  else if(e.getActionCommand().equals("Fit into one Range"))
	  {
	    fitIntoOneRange();
	  }
	  else if(e.getActionCommand().equals("Remove")) // cuttingplane
	  {
	    if(cp_list.getSelectedIndex()>=0)
	    {
	      cp_planes.removeElementAt(cp_list.getSelectedIndex());
	      cp_list.remove(cp_list.getSelectedIndex());
	    }
	  }
	  else if(e.getActionCommand().equals("Add")) // cuttingplane
	  {
	    CuttingPlane newcp = new CuttingPlane();
	    addCuttingPlane(newcp);
	    new CuttingPlaneDlg(this, newcp);
	  }
	  else if(e.getActionCommand().equals("Edit")) // cuttingplane
	  {
	    if(cp_list.getSelectedIndex()>=0)
	    {
	      new CuttingPlaneDlg(this, (CuttingPlane) cp_planes.elementAt(cp_list.getSelectedIndex()) );
	    }
	  }
	  else if(e.getActionCommand().equals(" Add")) // marker
	  {
	    if(go.technique == GraphOutput.EPC)
	    {
	      MarkerEPC newmarker = new MarkerEPC(go.style.getUniqueNumber(), go);
          addMarker(newmarker);
	      new MarkerDlg(this, newmarker);
	    }
	    else if(go.technique == GraphOutput.LWW)
	    {
	      MarkerLWW newmarker = new MarkerLWW(go.style.getUniqueNumber(), go);
          addMarker(newmarker);
	      new MarkerDlg(this, newmarker);
	    }
	    else if(go.technique == GraphOutput.TDC)
	    {
	      MarkerTDC newmarker = new MarkerTDC(go.style.getUniqueNumber(), go);
          addMarker(newmarker);
	      new MarkerDlg(this, newmarker);
	    }
//extend
	  }
	  else if(e.getActionCommand().equals(" Edit")) // marker
	  {
	    if(hl_list.getSelectedIndex()>=0)
	    {
	      Object marker = new Object();
	      if(go.technique == GraphOutput.EPC)
	       marker = (MarkerEPC) hl_markers.elementAt(hl_list.getSelectedIndex());
	      else if(go.technique == GraphOutput.LWW)
	       marker = (MarkerLWW) hl_markers.elementAt(hl_list.getSelectedIndex());
	      else if(go.technique == GraphOutput.TDC)
	       marker = (MarkerTDC) hl_markers.elementAt(hl_list.getSelectedIndex());
//extend
	      new MarkerDlg(this, marker);
	    }
	  }
	  else if(e.getActionCommand().equals(" Remove")) // marker
	  {
	    if(hl_list.getSelectedIndex()>=0)
	    {
	      hl_markers.removeElementAt(hl_list.getSelectedIndex());
	      hl_list.remove(hl_list.getSelectedIndex());
	    }
	  }
	  else if(e.getActionCommand().equals("move up"))
	  {
	    if(ma_list1.getSelectedIndex()>0)
	    {
	      int index = ma_list1.getSelectedIndex();
	      String item = new String(ma_list1.getItem(ma_list1.getSelectedIndex()));
          ma_list1.remove(index);
          ma_list1.add(item, index-1);
          ma_list1.select(index-1);
	    }
	  }
	  else if(e.getActionCommand().equals("move down"))
	  {
	    if(ma_list1.getSelectedIndex()<ma_list1.getItemCount()-1)
	    {
	      int index = ma_list1.getSelectedIndex();
	      String item = new String(ma_list1.getItem(ma_list1.getSelectedIndex()));
          ma_list1.remove(index);
          ma_list1.add(item, index+1);
          ma_list1.select(index+1);
	    }
	  }
	  else if(e.getActionCommand().equals("hide ->"))
	  {
	    if(ma_list1.getSelectedIndex()>=0)
	    {
	      int index = ma_list1.getSelectedIndex();
	      String item = new String(ma_list1.getItem(ma_list1.getSelectedIndex()));
          ma_list1.remove(index);
          ma_list2.add(item);
          ma_list2.select(ma_list2.getItemCount()-1);
	    }
	  }
	  else if(e.getActionCommand().equals("<- show"))
	  {
	    if(ma_list2.getSelectedIndex()>=0)
	    {
	      int index = ma_list2.getSelectedIndex();
	      String item = new String(ma_list2.getItem(ma_list2.getSelectedIndex()));
          ma_list2.remove(index);
          ma_list1.add(item);
          ma_list1.select(ma_list1.getItemCount()-1);
	    }
	  }
	  else if(e.getSource() instanceof Button)
	  {
         new ColorDlg(this, (Button)e.getSource());
	  }

	}

   /**
    * is called if the state of the Checkbox is changed
	*/
   public void itemStateChanged(ItemEvent e)
   {
     if(((String)e.getItem())=="Scaling")
       cl.show(clp, "scaling");
     else if(((String)e.getItem())=="Colors")
       cl.show(clp, "colors");
     else if(((String)e.getItem())=="Cutting-Planes")
       cl.show(clp, "cutting-planes");
     else if(((String)e.getItem())=="Mapping")
       cl.show(clp, "mapping");
     else if(((String)e.getItem())=="Animation")
       cl.show(clp, "animation");
     else if(((String)e.getItem())=="Highlighting")
       cl.show(clp, "highlighting");

   }

   /**
    * adds a new CuttingPlane to the list and to
    * the vector cp_planes
    */
   public void addCuttingPlane(CuttingPlane cp)
   {
     cp_planes.addElement(cp);
     cp_list.add(cp.description);
   }

   /**
    * adds a new Marker to the list and to
    * the vector hl_markers
    */
   public void addMarker(Object marker)
   {
     if(marker instanceof MarkerEPC)
     {
       hl_markers.addElement((MarkerEPC)marker);
       hl_list.add(((MarkerEPC)marker).getDescription());
     }
     if(marker instanceof MarkerLWW)
     {
       hl_markers.addElement((MarkerLWW)marker);
       hl_list.add(((MarkerLWW)marker).getDescription());
     }
     if(marker instanceof MarkerTDC)
     {
       hl_markers.addElement((MarkerTDC)marker);
       hl_list.add(((MarkerTDC)marker).getDescription());
     }
//extend
   }


   /**
    * creates a cardlayout including all 6 cards for
    * the options-dialog
    */
   private Panel createCardPanel()
   {
     Font bigFont = new Font("Dialog", Font.BOLD, 14);

     clp = new Panel();
     cl = new CardLayout();
     clp.setLayout(cl);

     // --------- scaling
     Panel sc_panel = new Panel();
     sc_panel.setLayout(new GridLayout(1,2));


     Panel pa = new Panel();
     pa.setLayout(new BorderLayout());

     ScrollPane sp = new ScrollPane();
     Panel p1 = new Panel();
     p1.setLayout(new GridLayout(go.scaling_v.size(), 1));
     for (int i=0; i<go.scaling_v.size(); i++)
     {
       Panel p = new Panel();
       p.setLayout(new FlowLayout());
       p.add(new Label(""+((String)go.variablenames_v.elementAt(i))+": min:", Label.RIGHT));
       TextField tf1 = new TextField(""+((Range)go.scaling_v.elementAt(i)).getMin3(), 5);
       sc_min_v.addElement(tf1);
       p.add(tf1);
       p.add(new Label("max:", Label.RIGHT));
       TextField tf2 = new TextField(""+((Range)go.scaling_v.elementAt(i)).getMax3(), 5);
       sc_max_v.addElement(tf2);
       p.add(tf2);
       p1.add(p);
     }

     sp.add(p1);
     sp.doLayout();

     Label l0 = new Label("Variable Scalings", Label.CENTER);
     l0.setFont(bigFont);
     pa.add("North", l0);
     pa.add("Center", sp);
     Button fior = new Button("Fit into one Range");
     fior.addActionListener(this);
     pa.add("South", fior);

     Panel pb = new Panel();
     pb.setLayout(new BorderLayout());

     Label l1 = new Label("Coordinate-Box Size", Label.CENTER);
     l1.setFont(bigFont);
     pb.add("North", l1);
     Panel pb1 = new Panel();
     pb1.setLayout(new FlowLayout());

     Panel pb11 = new Panel();
     pb11.add(new Label("x: "));
     double sc_dx = (new Integer( (new Double((1000.0*go.coordBoxScale.x))).intValue() )).doubleValue()/1000.0;
     sc_x = new TextField(""+sc_dx, 5);
     pb11.add(sc_x);
     Panel pb12 = new Panel();
     pb12.add(new Label("y: "));
     double sc_dy = (new Integer( (new Double((1000.0*go.coordBoxScale.y))).intValue() )).doubleValue()/1000.0;
     sc_y = new TextField(""+sc_dy, 5);
     pb12.add(sc_y);
     Panel pb13 = new Panel();
     pb13.add(new Label("z: "));
     double sc_dz = (new Integer( (new Double((1000.0*go.coordBoxScale.z))).intValue() )).doubleValue()/1000.0;
     sc_z = new TextField(""+sc_dz, 5);
     pb13.add(sc_z);

     pb1.add(pb11);
     pb1.add(pb12);
     pb1.add(pb13);

     pb.add("Center", pb1);

     sc_panel.add(pa);
     sc_panel.add(pb);

     clp.add("scaling", sc_panel);

     // --------- colors

     Panel co_panel = new Panel();
     co_panel.setLayout(new GridLayout(1,2));

     Panel copa = new Panel();
     copa.setLayout(new BorderLayout());
     Label col1 = new Label("Dimensions", Label.CENTER);
     col1.setFont(bigFont);
     copa.add("North", col1);

     Panel cop1 = new Panel();
     cop1.setLayout(new GridLayout(go.dimcnt,1));
     for(int i=0; i<go.dimcnt; i++)
     {
       Panel p = new Panel();
       p.setLayout(new FlowLayout());
       p.add(new Label(""+((String)go.variablenames_v.elementAt(i))+":    Color:"));
       Button b = new Button("   Change   ");

       Color3f col = new Color3f();
       go.style.getAppearance(i).getMaterial().getAmbientColor(col);

       b.setBackground(new Color(col.x, col.y, col.z));
       b.addActionListener(this);
       co_col.addElement(b);
       p.add(b);

       cop1.add(p);
     }

     ScrollPane pane = new ScrollPane();
     pane.add(cop1);
     copa.add("Center", pane);

     Panel copb = new Panel();
     copb.setLayout(new BorderLayout());
     Label col2 = new Label("Graph Properties", Label.CENTER);
     col2.setFont(bigFont);
     copb.add("North", col2);

     Panel copb_1 = new Panel();
     copb_1.setLayout(new BorderLayout());
     Panel copb_p0 = new Panel();
     co_showgrid = new Checkbox("show grid", go.style.showgrid);
     co_showlines = new Checkbox("show additional lines", go.style.showlines);
     copb_p0.add(co_showgrid);
     copb_p0.add(co_showlines);
     copb_1.add("North", copb_p0);
     co_fraction = new TextField("" + ((int)(100.0-(100.0*go.style.space_frac))));
     if(go.technique == GraphOutput.EPC)
     {
       Panel copb_p = new Panel();
       copb_p.add(new Label("% of width used by stripes", Label.RIGHT));
       copb_p.add(co_fraction);
       copb_1.add("Center", copb_p);
     }
    co_showplanes = new Checkbox("show planes", go.style.showplanes);
    co_showsurface = new Checkbox("show surface", go.style.showsurface);
    co_transparency = new TextField("" + (float)((int)(100.0f*go.style.sur_transparency))/100.0f);
    if(go.technique == GraphOutput.TDC)
     {
       Panel copb_p = new Panel();
       copb_p.add(co_showplanes);
       copb_p.add(co_showsurface);
       Panel cot_p = new Panel();
       cot_p.setLayout(new GridLayout(1,2));
       cot_p.add(new Label("Transparency of surface:", Label.RIGHT));
       cot_p.add(co_transparency);
       copb_1.add("South", cot_p);
       copb_1.add("Center", copb_p);
     }
     copb.add("Center", copb_1);

     co_panel.add(copa);
     co_panel.add(copb);
     clp.add("colors", co_panel);

     // --------- cutting-planes
     Panel cp_panel = new Panel();
     cp_panel.setLayout(new BorderLayout());
     Label cp_l1 = new Label("Cutting Planes", Label.CENTER);
     cp_l1.setFont(bigFont);
     cp_panel.add("North", cp_l1);

     cp_planes = (Vector)go.style.cuttingplane_v.clone();
     cp_list = new java.awt.List();
     for (int i=0; i<cp_planes.size(); i++)
     {
       cp_list.add ( ((CuttingPlane)cp_planes.elementAt(i)).description );
     }
     cp_panel.add("Center", cp_list);
     Panel cp_p = new Panel();
     Button cp_b1 = new Button("Edit");
     cp_b1.addActionListener(this);
     Button cp_b2 = new Button("Add");
     cp_b2.addActionListener(this);
     Button cp_b3 = new Button("Remove");
     cp_b3.addActionListener(this);
     cp_p.add(cp_b1);
     cp_p.add(cp_b2);
     cp_p.add(cp_b3);
     cp_panel.add("South", cp_p);

     clp.add("cutting-planes", cp_panel);

     // --------- mapping
     Panel ma_panel = new Panel();
     ma_panel.setLayout(new BorderLayout());

     Label ma_l1 = new Label("Mapping of variables", Label.CENTER);
     ma_l1.setFont(bigFont);

     Panel ma_p1 = new Panel();
     ma_p1.setLayout(new BorderLayout());
     ma_p1.add("North", new Label("Order of viewed variables", Label.CENTER));
     ma_list1 = new java.awt.List();
     for (int i=0; i<go.dimcnt; i++)
       ma_list1.add((String)go.variablenames_v.elementAt(i));
     ma_p1.add("Center", ma_list1);
     Panel ma_p2 = new Panel();
     ma_p2.setLayout(new BorderLayout());
     ma_p2.add("North", new Label("  ", Label.CENTER));
     Panel ma_p21 = new Panel();
     ma_p21.setLayout(new GridLayout(2,2, 110, 10));
     Button ma_b1 = new Button("move up");
     Button ma_b2 = new Button("hide ->");
     Button ma_b3 = new Button("move down");
     Button ma_b4 = new Button("<- show");
     ma_b1.addActionListener(this);
     ma_b2.addActionListener(this);
     ma_b3.addActionListener(this);
     ma_b4.addActionListener(this);
     ma_p21.add(ma_b1);
     ma_p21.add(ma_b2);
     ma_p21.add(ma_b3);
     ma_p21.add(ma_b4);
     ma_p2.add("Center", ma_p21);
     Panel ma_p3 = new Panel();
     ma_p3.setLayout(new BorderLayout());
     ma_p3.add("North", new Label("Hidden variables", Label.CENTER));
     ma_list2 = new java.awt.List();
     for (int i=go.dimcnt; i<go.variablenames_v.size(); i++)
         ma_list2.add((String)go.variablenames_v.elementAt(i));
     ma_p3.add("Center", ma_list2);

     ma_panel.add("North", ma_l1);
     ma_panel.add("West", ma_p1);
     ma_panel.add("Center", ma_p2);
     ma_panel.add("East", ma_p3);

     clp.add("mapping", ma_panel);

     // --------- animation
     Panel an_panel = new Panel();
     an_panel.add(new Label("in animation-panel"));
     clp.add("animation", an_panel);

     // --------- highlighting
     Panel hl_panel = new Panel();
     hl_panel.setLayout(new BorderLayout());
     Label hl_l1 = new Label("Highlighting markers", Label.CENTER);
     hl_l1.setFont(bigFont);
     hl_panel.add("North", hl_l1);

     hl_markers = (Vector)go.style.marker_v.clone();
     hl_list = new java.awt.List();
     for (int i=0; i<hl_markers.size(); i++)
     {
       if(go.technique == GraphOutput.EPC)
         hl_list.add ( ((MarkerEPC)hl_markers.elementAt(i)).getDescription() );
       if(go.technique == GraphOutput.LWW)
         hl_list.add ( ((MarkerLWW)hl_markers.elementAt(i)).getDescription() );
       if(go.technique == GraphOutput.TDC)
         hl_list.add ( ((MarkerTDC)hl_markers.elementAt(i)).getDescription() );
//extend
     }
     hl_panel.add("Center", hl_list);
     Panel hl_p = new Panel();
     Button hl_b1 = new Button(" Edit");
     hl_b1.addActionListener(this);
     Button hl_b2 = new Button(" Add");
     hl_b2.addActionListener(this);
     Button hl_b3 = new Button(" Remove");
     hl_b3.addActionListener(this);
     hl_p.add(hl_b1);
     hl_p.add(hl_b2);
     hl_p.add(hl_b3);
     hl_panel.add("South", hl_p);



     clp.add("highlighting", hl_panel);


     return clp;
   }

   /**
    * extracts the new input data out of the dialog
    * components, updates the values in graphoutput
    * and repaints it
    */
   private void updateGraphoutput()
   {
     // ---------- scaling - section
     int sv_size = go.scaling_v.size();
     go.scaling_v=new Vector();
     for (int i=0; i<sv_size; i++)
     {
       Range r = new Range( (new Double( ((TextField)sc_min_v.elementAt(i)).getText())).doubleValue(),
                            (new Double(((TextField)sc_max_v.elementAt(i)).getText())).doubleValue() );
       go.scaling_v.addElement(r);
     }

     go.setBoxSize( (new Double(sc_x.getText())).doubleValue(),
                    (new Double(sc_y.getText())).doubleValue(),
                    (new Double(sc_z.getText())).doubleValue() );

     // ----------- coloring - section

     for( int i=0; i<go.dimcnt; i++)
     {
        Color c = ((Button)co_col.elementAt(i)).getBackground();

        Color3f objColor = new Color3f( (float)c.getRed()/255.0f,
                                        (float)c.getGreen()/255.0f,
                                        (float)c.getBlue()/255.0f );
        Appearance app = new Appearance();
        Material mat = new Material(objColor, new Color3f(0.0f,0.0f,0.0f), objColor,
	                                              new Color3f(1.0f,1.0f,1.0f), 80.0f);
        mat.setCapability(Material.ALLOW_COMPONENT_READ);
        mat.setCapability(Material.ALLOW_COMPONENT_WRITE);
        app.setMaterial(mat);
        PolygonAttributes pa = new PolygonAttributes();
        pa.setCullFace(pa.CULL_NONE);
        app.setPolygonAttributes(pa);
        if(i==0) go.style.initAppearance(app);
        else go.style.addAppearance(app);

     }

     go.style.showgrid = co_showgrid.getState();
     go.style.showlines = co_showlines.getState();
     go.style.showplanes = co_showplanes.getState();
     go.style.showsurface = co_showsurface.getState();
     go.style.sur_transparency = new Float(co_transparency.getText()).floatValue();
	 if(go.technique == GraphOutput.EPC)
       go.style.space_frac = 1.0-(new Double(co_fraction.getText())).doubleValue()/100.0;

     // ------------ cuttingplanes - section
     go.style.cuttingplane_v = new Vector();
     for(int i=0; i<cp_planes.size(); i++)
     {
       try {
         go.style.addCuttingPlane(go.graph, (CuttingPlane)cp_planes.elementAt(i));
       } catch(javax.media.j3d.MultipleParentException e) { }
     }

     // ------------- mapping - section
     int newdimcnt = ma_list1.getItemCount();
     go.mapping_v = new Vector();
     for (int i=0; i<newdimcnt; i++)
       go.mapping_v.add((String)ma_list1.getItem(i));
     for (int i=0; i<ma_list2.getItemCount(); i++)
       go.mapping_v.add((String)ma_list2.getItem(i));

     go.dimcnt = newdimcnt;



     // ------------- animation - section

     // ------------- highlighting - section
     go.style.marker_v = new Vector();
     for(int i=0; i<hl_markers.size(); i++)
     {
       try {
         if (go.technique == GraphOutput.EPC)
           go.style.marker_v.add((MarkerEPC)hl_markers.elementAt(i));
         else if (go.technique == GraphOutput.LWW)
           go.style.marker_v.add((MarkerLWW)hl_markers.elementAt(i));
         else if (go.technique == GraphOutput.TDC)
           go.style.marker_v.add((MarkerTDC)hl_markers.elementAt(i));
//extend
       } catch(javax.media.j3d.MultipleParentException e) { }
     }


     go.display();

   }

   /**
    * fills the list of cuttingplanes
    */
   public void updateList()
   {
     cp_list.removeAll();
     for (int i=0; i<cp_planes.size(); i++)
     {
       cp_list.add ( ((CuttingPlane)cp_planes.elementAt(i)).description );
     }

   }

   /**
    * fills the list of markers
    */
   public void updateMarkerList()
   {
     hl_list.removeAll();
     for (int i=0; i<hl_markers.size(); i++)
     {
       if(go.technique==GraphOutput.EPC)
         hl_list.add ( ((MarkerEPC)hl_markers.elementAt(i)).getDescription() );
       if(go.technique==GraphOutput.LWW)
         hl_list.add ( ((MarkerLWW)hl_markers.elementAt(i)).getDescription() );
       if(go.technique==GraphOutput.TDC)
         hl_list.add ( ((MarkerTDC)hl_markers.elementAt(i)).getDescription() );
//extend
     }

   }


   /**
    * creates a Range object, in which all of the
    * Ranges of the variables fit in
    */
   private void fitIntoOneRange()
   {
     Range r = new Range();

     for (int i=0; i<go.dimcnt; i++)
     {
       r.checkRange( ((Range)go.scaling_v.elementAt(i)).getMin() );
       r.checkRange( ((Range)go.scaling_v.elementAt(i)).getMax() );
     }

     for (int i=0; i<go.dimcnt; i++)
     {
       ((TextField)sc_min_v.elementAt(i)).setText(""+r.getMin3());
       ((TextField)sc_max_v.elementAt(i)).setText(""+r.getMax3());
     }


   }




    public void windowActivated(WindowEvent e) {}
    public void windowClosed(WindowEvent e) {}
    public void windowClosing(WindowEvent e)
	{
      setVisible(false);
	  dispose();
	}
    public void windowDeactivated(WindowEvent e) {}
    public void windowDeiconified(WindowEvent e) {}
    public void windowIconified(WindowEvent e) {}
    public void windowOpened(WindowEvent e) {}


}

