package graphoutput;
import com.sun.j3d.utils.behaviors.mouse.*;
import com.sun.j3d.utils.behaviors.picking.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.media.j3d.*;
import javax.vecmath.*;

/**
 * A mouse behavior that allows user to pick and translate scene graph objects.
 * Common usage: 1. Create your scene graph. 2. Create this behavior with
 * the root and canvas. See PickRotateBehavior for more details. 
 */

public class PickTranslateBehavior3 extends PickMouseBehavior implements MouseBehaviorCallback {
  private static final double DRAG_FACTOR = 0.002;

  MouseTranslate3 translate;
  int pickMode = PickObject.USE_BOUNDS;
  private PickingCallback callback = null;
  private TransformGroup currentTG;

  /**
   * Creates a pick/translate behavior that waits for user mouse events for
   * the scene graph. This method has its pickMode set to BOUNDS picking. 
   * @param root   Root of your scene graph.
   * @param canvas Java 3D drawing canvas.
   * @param bounds Bounds of your scene.
   **/

  public PickTranslateBehavior3(BranchGroup root, Canvas3D canvas, Bounds bounds, GraphOutput go){
    super(canvas, root, bounds);
    translate = new MouseTranslate3(MouseBehavior.MANUAL_WAKEUP, go);
    translate.setFactor(DRAG_FACTOR);
    translate.setTransformGroup(currGrp);
    currGrp.addChild(translate);
    translate.setSchedulingBounds(bounds);
    this.setSchedulingBounds(bounds);
  }

  /**
   * Creates a pick/translate behavior that waits for user mouse events for
   * the scene graph.
   * @param root   Root of your scene graph.
   * @param canvas Java 3D drawing canvas.
   * @param bounds Bounds of your scene.
   * @param pickMode specifys PickObject.USE_BOUNDS or PickObject.USE_GEOMETRY.
   * Note: If pickMode is set to PickObject.USE_GEOMETRY, all geometry object in
   * the scene graph that allows pickable must have its ALLOW_INTERSECT bit set.
   **/

  public PickTranslateBehavior3(BranchGroup root, Canvas3D canvas, Bounds bounds,
 			       int pickMode, GraphOutput go){
    super(canvas, root, bounds);
    translate = new MouseTranslate3(MouseBehavior.MANUAL_WAKEUP, go);
    translate.setFactor(DRAG_FACTOR);
    translate.setTransformGroup(currGrp);
    currGrp.addChild(translate);
    translate.setSchedulingBounds(bounds);
    this.setSchedulingBounds(bounds);
    this.pickMode = pickMode;
  }

  /**
   * Sets the pickMode component of this PickTranslateBehavior3 to the value of
   * the passed pickMode.
   * @param pickMode the pickMode to be copied.
   **/

  public void setPickMode(int pickMode) {
    this.pickMode = pickMode;
  }

 /**
   * Return the pickMode component of this PickTranslaeBehavior.
   **/

  public int getPickMode() {
    return pickMode;
  }

  /**
   * Update the scene to manipulate any nodes. This is not meant to be
   * called by users. Behavior automatically calls this. You can call 
   * this only if you know what you are doing.
   * 
   * @param xpos Current mouse X pos.
   * @param ypos Current mouse Y pos.
   **/
  public void updateScene(int xpos, int ypos){
    TransformGroup tg = null;
        
    if (!mevent.isAltDown() && mevent.isMetaDown()){
      
      tg =(TransformGroup)pickScene.pickNode(pickScene.pickClosest(xpos, ypos, pickMode),
					     PickObject.TRANSFORM_GROUP, 2);
      //Check for valid selection. 
      if ((tg != null) && 
	  (tg.getCapability(TransformGroup.ALLOW_TRANSFORM_READ)) && 
	  (tg.getCapability(TransformGroup.ALLOW_TRANSFORM_WRITE))){
	
	translate.setTransformGroup(tg);
	translate.wakeup();
	currentTG = tg;
      } else if (callback!=null)
	callback.transformChanged( PickingCallback.NO_PICK, null );
    }
    
  }

  /**
    * Callback method from MouseTranslate3
    * This is used when the Picking callback is enabled
    */
  public void transformChanged( int type, Transform3D transform ) {
      callback.transformChanged( PickingCallback.TRANSLATE, currentTG );
  }

  /**
    * Register the class @param callback to be called each
    * time the picked object moves
    */
  public void setupCallback( PickingCallback callback ) {
      this.callback = callback;
      if (callback==null)
          translate.setupCallback( null );
      else
          translate.setupCallback( this );
  }

}

